%% Copyright (c) 2017 Brook Milligan.
%%
%% This file is part of the GraphModel Library.
%%
%%     The GraphModel Library is free software: you can redistribute
%%     it and/or modify it under the terms of the GNU Affero General
%%     Public License as published by the Free Software Foundation,
%%     either version 3 of the License, or (at your option) any later
%%     version.
%%
%%     The GraphModel Library is distributed in the hope that it will
%%     be useful, but WITHOUT ANY WARRANTY; without even the implied
%%     warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
%%     PURPOSE.  See the GNU Affero General Public License for more
%%     details.
%%
%%     You should have received a copy of the GNU Affero General
%%     Public License along with the GraphModel Library.  If not, see
%%     <http://www.gnu.org/licenses/>.
%%

\page page_simplified_class_hierarchy Simplified class hierarchy

\tableofcontents


\section simplified_vertex_class_hierarchy Simplified vertex class hierarchy

Graph models may be composed of random variables and factors,
which are represented by members of a class hierarchy.  Each class
in the hierarchy corresponds to one of the basic concepts involved
in graph models:

- Expression: an expression can be evaluated

- Factor: a factor is an expression that evaluates a \c pdf() member
  function

- Scalar factor: a scalar factor is a factor with a \c cdf() member
  function

- Distribution: a distribution is a factor with a \c sample() member
  function

- Scalar distribution: a scalar distribution is a distribution with a
  \c cdf() member function

- Random variable: a random variable is an expression that evaluates a
  \c variate() member function.  In addition, its interface is much
  broader.

- Scalar random variable: a scalar random variable is a random
  variable with a \c cdf() member function.

This simplified class hierarchy is illustrated below.

\dotfile docs/class_hierarchy.dot "Simplified vertex class hierarchy"

The actual class hierarchy is more fine-grained than implied by
the figure.  Each of the basic components that compose these
classes is itself its own class.  For example, the class \c
pdf_base defines the interface for classes implementing the \c
pdf() member function.  These components are composed by base
classes that correspond to each set of functionality introduced
above.  In turn, the user-facing classes introduced above are
derived from from their corresponding base classes.


\section simplified_policy_concept_hierarchy Simplified policy concept hierarchy

The behavior of many of the vertex types is controlled by a strategy
that can be replaced at runtime with the \c reset_strategy() member
function.  The argument to \c reset_strategy() may be any class that
models the appropriate concept.  These concepts are also organized
into a class hierarchy that parallels that of the vertex types
themselves.  The simplified concept hierarchy is illustrated below.

\dotfile docs/policy_concept_hierarchy.dot "Simplified policy concept hierarchy"
  
