/*
 * Copyright (c) 2016,2017 Brook Milligan.
 *
 * This file is part of the GraphModel Library.
 *
 *     The GraphModel Library is free software: you can redistribute
 *     it and/or modify it under the terms of the GNU Affero General
 *     Public License as published by the Free Software Foundation,
 *     either version 3 of the License, or (at your option) any later
 *     version.
 *
 *     The GraphModel Library is distributed in the hope that it will
 *     be useful, but WITHOUT ANY WARRANTY; without even the implied
 *     warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
 *     PURPOSE.  See the GNU Affero General Public License for more
 *     details.
 *
 *     You should have received a copy of the GNU Affero General
 *     Public License along with the GraphModel Library.  If not, see
 *     <http://www.gnu.org/licenses/>.
 */

/// \file

#ifndef DATAFRAME_READ_DATAFRAME_HPP
#define DATAFRAME_READ_DATAFRAME_HPP 1

#include <boost/filesystem/fstream.hpp>
#include <dataframe/dataframe_io.hpp>


namespace dataframe
{

  namespace detail
  {

    struct read_dataframe_helper
    {
      template < typename Path, typename ... Manipulators >
      static auto apply (Path path, Manipulators ... manipulators)
      {
	using ifstream_type = boost::filesystem::ifstream;
	ifstream_type ifstream (path);
	dataframe<> dataframe;
	apply_manipulators(ifstream,manipulators...);
	ifstream >> dataframe;
	return dataframe;
      }

    private:
      template < typename Stream >
      static void apply_manipulators (Stream&) {}

      template < typename Manipulator, typename ... Manipulators >
      static void apply_manipulators (std::istream& is, Manipulator manipulator, Manipulators ... manipulators)
      {
	is >> manipulator;
	apply_manipulators(is,manipulators...);
      }

      template < typename Manipulator, typename ... Manipulators >
      static void apply_manipulators (std::ostream& os, Manipulator manipulator, Manipulators ... manipulators)
      {
	os << manipulator;
	apply_manipulators(os,manipulators...);
      }
    };

  } // namespace detail

  template < typename Path, typename ... Manipulators >
  auto read_dataframe (Path path, Manipulators ... manipulators)
  { return detail::read_dataframe_helper::apply(path,manipulators...); }
  
} // namespace dataframe

#endif
