/*
 * Copyright (c) 2016,2017 Brook Milligan.
 *
 * This file is part of the GraphModel Library.
 *
 *     The GraphModel Library is free software: you can redistribute
 *     it and/or modify it under the terms of the GNU Affero General
 *     Public License as published by the Free Software Foundation,
 *     either version 3 of the License, or (at your option) any later
 *     version.
 *
 *     The GraphModel Library is distributed in the hope that it will
 *     be useful, but WITHOUT ANY WARRANTY; without even the implied
 *     warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
 *     PURPOSE.  See the GNU Affero General Public License for more
 *     details.
 *
 *     You should have received a copy of the GNU Affero General
 *     Public License along with the GraphModel Library.  If not, see
 *     <http://www.gnu.org/licenses/>.
 */

/// \file

#ifndef DEBUGGING_CLASS_NAME_HPP
#define DEBUGGING_CLASS_NAME_HPP 1

#include <string>
#include <typeinfo>
#include <boost/core/demangle.hpp>
#include <boost/algorithm/string/find.hpp>

namespace debugging
{

  struct class_name
  {
    template < typename T >
    static std::string apply ()
    {
      using boost::core::demangle;
      using boost::algorithm::find_first;
      using boost::algorithm::find_last;
      auto name = demangle(typeid(T).name());
      std::string scoped_name (name.begin(),find_first(name,"<").begin());
      std::string class_name (find_last(scoped_name,"::").end(),scoped_name.end());
      return class_name.empty() ? scoped_name : class_name;
    }
  };

  struct full_class_name
  {
    template < typename T >
    static std::string apply ()
    {
      using boost::core::demangle;
      return demangle(typeid(T).name());
    }
  };

} // namespace debugging

#endif
