/*
 * Copyright (c) 2016,2017 Brook Milligan.
 *
 * This file is part of the GraphModel Library.
 *
 *     The GraphModel Library is free software: you can redistribute
 *     it and/or modify it under the terms of the GNU Affero General
 *     Public License as published by the Free Software Foundation,
 *     either version 3 of the License, or (at your option) any later
 *     version.
 *
 *     The GraphModel Library is distributed in the hope that it will
 *     be useful, but WITHOUT ANY WARRANTY; without even the implied
 *     warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
 *     PURPOSE.  See the GNU Affero General Public License for more
 *     details.
 *
 *     You should have received a copy of the GNU Affero General
 *     Public License along with the GraphModel Library.  If not, see
 *     <http://www.gnu.org/licenses/>.
 */

/// \file

#ifndef GRAPH_MODEL_GRAPH_MODEL_HPP
#define GRAPH_MODEL_GRAPH_MODEL_HPP 1

#include <graph_model/graph_model_fwd.hpp>
#include <graph_model/graph/adjacency_graph_base.hpp>
#include <graph_model/graph/add_connected_components_base.hpp>
#include <graph_model/graph/edge_inserter.hpp>
#include <graph_model/graph/edge_list_graph_base.hpp>
#include <graph_model/graph/incidence_graph_base.hpp>
#include <graph_model/graph/streamable_base.hpp>
#include <graph_model/graph/vertex_category_property_map_fwd.hpp>
#include <graph_model/graph/vertex_gid_property_map_fwd.hpp>
#include <graph_model/graph/vertex_identifier_property_map_fwd.hpp>
#include <graph_model/graph/vertex_inserter.hpp>
#include <graph_model/graph/vertex_list_graph_base.hpp>
#include <graph_model/graph_traits.hpp>
#include <graph_model/type_traits/is_incidence_graph.hpp>
#include <mpl/enable_constructor.hpp>

namespace graph_model
{

  template <>
  class graph_model< tag::incidence_graph >
    : public graph::incidence_graph_base< graph_model<tag::incidence_graph>
    , graph::adjacency_graph_base<graph_model<tag::incidence_graph>
    , graph::streamable_base< graph_model<tag::incidence_graph>,tag::incidence_graph
    > > >
  {
    using self = graph_model< tag::incidence_graph >;
    using streamable_base = graph::streamable_base<self,tag::incidence_graph>;
    using adjacency_graph_base = graph::adjacency_graph_base<self,streamable_base>;
    using incidence_graph_base = graph::incidence_graph_base<self,adjacency_graph_base>;
    using base = incidence_graph_base;

  public:
    template < typename ... Args
	       , typename = mpl::enable_constructor_t<self,Args...>
	       >
    graph_model (Args&& ... args)
      : base(std::forward<Args>(args)...)
    {}
  };

  template <>
  class graph_model< tag::vertex_list_graph >
    : public graph::vertex_list_graph_base<graph_model<tag::vertex_list_graph>,tag::vertex_list_graph
    , graph::incidence_graph_base<graph_model<tag::vertex_list_graph>
    , graph::adjacency_graph_base<graph_model<tag::vertex_list_graph>
    , graph::add_connected_components_base<graph_model<tag::vertex_list_graph>,tag::vertex_list_graph
    , graph::streamable_base<graph_model<tag::vertex_list_graph>,tag::vertex_list_graph
    > > > > >
  {
    using self = graph_model;
    using tag = tag::vertex_list_graph;
    using streamable_base = graph::streamable_base<self,tag>;
    using add_connected_components_base = graph::add_connected_components_base<self,tag,streamable_base>;
    using adjacency_graph_base = graph::adjacency_graph_base<self,add_connected_components_base>;
    using incidence_graph_base = graph::incidence_graph_base<self,adjacency_graph_base>;
    using vertex_list_graph_base = graph::vertex_list_graph_base<self,tag,incidence_graph_base>;
    using base = vertex_list_graph_base;

  public:
    template < typename ... Args
	       , typename = mpl::enable_constructor_t<self,Args...>
	       >
    graph_model (Args&& ... args)
      : base(std::forward<Args>(args)...)
    {}
  };

  template <>
  class graph_model< tag::edge_list_graph >
    : public graph::edge_list_graph_base<graph_model<tag::edge_list_graph>,tag::edge_list_graph
    , graph::incidence_graph_base<graph_model<tag::vertex_list_graph>
    , graph::adjacency_graph_base<graph_model<tag::edge_list_graph>
    , graph::add_connected_components_base<graph_model<tag::edge_list_graph>,tag::edge_list_graph
    , graph::streamable_base<graph_model<tag::edge_list_graph>,tag::edge_list_graph
    > > > > >
  {
    using self = graph_model;
    using tag = tag::edge_list_graph;
    using streamable_base = graph::streamable_base<self,tag>;
    using add_connected_components_base = graph::add_connected_components_base<self,tag,streamable_base>;
    using adjacency_graph_base = graph::adjacency_graph_base<self,add_connected_components_base>;
    using incidence_graph_base = graph::incidence_graph_base<self,adjacency_graph_base>;
    using edge_list_graph_base = graph::edge_list_graph_base<self,tag,incidence_graph_base>;
    using base = edge_list_graph_base;

  public:
    template < typename ... Args
	       , typename = mpl::enable_constructor_t<self,Args...>
	       >
    graph_model (Args&& ... args)
      : base(std::forward<Args>(args)...)
    {}
  };

  template <>
  class graph_model < tag::vertex_and_edge_list_graph >
    : public graph::vertex_list_graph_base<graph_model<tag::vertex_and_edge_list_graph>,tag::vertex_and_edge_list_graph
    , graph::edge_list_graph_base<graph_model<tag::vertex_and_edge_list_graph>,tag::vertex_and_edge_list_graph
    , graph::incidence_graph_base<graph_model<tag::vertex_list_graph>
    , graph::adjacency_graph_base<graph_model<tag::vertex_and_edge_list_graph>
    , graph::add_connected_components_base<graph_model<tag::vertex_and_edge_list_graph>,tag::vertex_and_edge_list_graph
    , graph::streamable_base<graph_model<tag::vertex_and_edge_list_graph>,tag::vertex_and_edge_list_graph
    > > > > > >
  {
    using self = graph_model;
    using tag = tag::vertex_and_edge_list_graph;
    using streamable_base = graph::streamable_base<self,tag>;
    using add_connected_components_base = graph::add_connected_components_base<self,tag,streamable_base>;
    using adjacency_graph_base = graph::adjacency_graph_base<self,add_connected_components_base>;
    using incidence_graph_base = graph::incidence_graph_base<self,adjacency_graph_base>;
    using edge_list_graph_base = graph::edge_list_graph_base<self,tag,incidence_graph_base>;
    using vertex_list_graph_base = graph::vertex_list_graph_base<self,tag,edge_list_graph_base>;
    using base = vertex_list_graph_base;

  public:
    graph_model () = default;

    template < typename Graph
	       , typename ... VertexDescriptors
	       , typename = std::enable_if_t<type_traits::is_incidence_graph_v<Graph>>
	       >
    graph_model (Graph const& g, typename boost::graph_traits<Graph>::vertex_descriptor source, VertexDescriptors&& ... others)
    { this->add_connected_components(g,source,std::forward<VertexDescriptors>(others)...); }

  public:			// XXX - for CRTP
    auto bfs_visitor() { return make_bfs_visitor(graph::make_vertex_inserter(*this),graph::make_edge_inserter(*this)); }
  };

  template < typename Tag, typename ... Args >
  auto make_graph_model (Args&& ... args)
  {
    using graph_model_type = graph_model<Tag>;
    return graph_model_type(std::forward<Args>(args)...);
  }

  template < typename ... Args >
  auto make_incidence_graph (Args&& ... args)
  { return make_graph_model<tag::incidence_graph>(std::forward<Args>(args)...); }

  template < typename ... Args >
  auto make_vertex_list_graph (Args&& ... args)
  { return make_graph_model<tag::vertex_list_graph>(std::forward<Args>(args)...); }

  template < typename ... Args >
  auto make_edge_list_graph (Args&& ... args)
  { return make_graph_model<tag::edge_list_graph>(std::forward<Args>(args)...); }

  template < typename ... Args >
  auto make_vertex_and_edge_list_graph (Args&& ... args)
  { return make_graph_model<tag::vertex_and_edge_list_graph>(std::forward<Args>(args)...); }

  namespace graph
  {

    template < typename GraphTag >
    class vertex_category_property_map< graph_model<GraphTag> >
    {
      using self = vertex_category_property_map;
      using graph_type = graph_model<GraphTag>;
      using graph_traits = boost::graph_traits<graph_type>;
      using pointer = observer_ptr<graph_type const>;

    public:
      vertex_category_property_map () = default;
      vertex_category_property_map (graph_type const& g)
	: p_(make_observer(g))
      {}

      template < typename VertexDescriptor >
      friend auto get (self const& map, VertexDescriptor u)
      { return u.vertex_category(); }

    public:
      using category = boost::readable_property_map_tag;
      using key_type = typename graph_traits::vertex_descriptor;
      using value_type = decltype(get(std::declval<self>(),std::declval<key_type>()));
      using reference = value_type;

    private:
      auto& graph () { return *p_; }
      auto& graph () const { return *p_; }

    private:
      pointer p_;
    };

    template < typename GraphTag >
    auto get (tag::vertex_category, graph_model<GraphTag> const& g)
    {
      using graph_type = graph_model<GraphTag>;
      using property_map = vertex_category_property_map<graph_type>;
      return property_map(g);
    }

    template < typename GraphTag, typename VertexDescriptor >
    auto get (tag::vertex_category, graph_model<GraphTag> const& g, VertexDescriptor u)
    {
      using graph_type = graph_model<GraphTag>;
      using property_map = vertex_category_property_map<graph_type>;
      return get(property_map(g),u);
    }

    template < typename GraphTag >
    class vertex_gid_property_map< graph_model<GraphTag> >
    {
      using self = vertex_gid_property_map;
      using graph_type = graph_model<GraphTag>;
      using graph_traits = boost::graph_traits<graph_type>;
      using pointer = observer_ptr<graph_type const>;

    public:
      vertex_gid_property_map () = default;
      vertex_gid_property_map (graph_type const& g)
	: p_(make_observer(g))
      {}

      template < typename VertexDescriptor >
      friend auto get (self const& map, VertexDescriptor u)
      { return u.id(); }

    public:
      using category = boost::readable_property_map_tag;
      using key_type = typename graph_traits::vertex_descriptor;
      using value_type = decltype(get(std::declval<self>(),std::declval<key_type>()));
      using reference = value_type;

    private:
      auto& graph () { return *p_; }
      auto& graph () const { return *p_; }
	
    private:
      pointer p_;
    };

    template < typename GraphTag >
    auto get (tag::vertex_gid, graph_model<GraphTag> const& g)
    {
      using graph_type = graph_model<GraphTag>;
      using property_map = vertex_gid_property_map<graph_type>;
      return property_map(g);
    }

    template < typename GraphTag, typename VertexDescriptor >
    auto get (tag::vertex_gid, graph_model<GraphTag> const& g, VertexDescriptor u)
    {
      using graph_type = graph_model<GraphTag>;
      using property_map = vertex_gid_property_map<graph_type>;
      return get(property_map(g),u);
    }

    template < typename GraphTag >
    class vertex_identifier_property_map< graph_model<GraphTag> >
    {
      using self = vertex_identifier_property_map;
      using graph_type = graph_model<GraphTag>;
      using graph_traits = boost::graph_traits<graph_type>;
      using pointer = observer_ptr<graph_type const>;

    public:
      vertex_identifier_property_map () = default;
      vertex_identifier_property_map (graph_type const& g)
	: p_(make_observer(g))
      {}

      template < typename VertexDescriptor >
      friend auto get (self const& map, VertexDescriptor u)
      { return u.identifier(); }

    public:
      using category = boost::readable_property_map_tag;
      using key_type = typename graph_traits::vertex_descriptor;
      using value_type = decltype(get(std::declval<self>(),std::declval<key_type>()));
      using reference = value_type;

    private:
      auto& graph () { return *p_; }
      auto& graph () const { return *p_; }

    private:
      pointer p_;
    };

    template < typename GraphTag >
    auto get (tag::vertex_identifier, graph_model<GraphTag> const& g)
    {
      using graph_type = graph_model<GraphTag>;
      using property_map = vertex_identifier_property_map<graph_type>;
      return property_map(g);
    }

    template < typename GraphTag, typename VertexDescriptor >
    auto get (tag::vertex_identifier, graph_model<GraphTag> const& g, VertexDescriptor u)
    {
      using graph_type = graph_model<GraphTag>;
      using property_map = vertex_identifier_property_map<graph_type>;
      return get(property_map(g),u);
    }

  } // namespace graph

} // namespace graph_model

#endif
