/*
 * Copyright (c) 2016,2017 Brook Milligan.
 *
 * This file is part of the GraphModel Library.
 *
 *     The GraphModel Library is free software: you can redistribute
 *     it and/or modify it under the terms of the GNU Affero General
 *     Public License as published by the Free Software Foundation,
 *     either version 3 of the License, or (at your option) any later
 *     version.
 *
 *     The GraphModel Library is distributed in the hope that it will
 *     be useful, but WITHOUT ANY WARRANTY; without even the implied
 *     warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
 *     PURPOSE.  See the GNU Affero General Public License for more
 *     details.
 *
 *     You should have received a copy of the GNU Affero General
 *     Public License along with the GraphModel Library.  If not, see
 *     <http://www.gnu.org/licenses/>.
 */

/// \file

#ifndef GRAPH_MODEL_TRAITS_OUT_EDGE_ITERATOR_HPP
#define GRAPH_MODEL_TRAITS_OUT_EDGE_ITERATOR_HPP 1

#include <graph_model/graph_tags.hpp>
#include <graph_model/traits/vertex_descriptor.hpp>
#include <graph_model/traits/vertex_observer_collection.hpp>
#include <graph_model/edge/out_edge_iterator.hpp>

namespace graph_model
{

  namespace traits
  {

    template < typename GraphTag >
    struct out_edge_iterator
    {
      using type = void;
    };

    template <>
    class out_edge_iterator< tag::incidence_graph >
    {
      using vertex_descriptor = vertex_descriptor_t<tag::incidence_graph>;
      using vertex_observer_collection = vertex_observer_collection_t<tag::incidence_graph>;
      using iterator = ::graph_model::edge::out_edge_iterator<vertex_descriptor,vertex_observer_collection>;
    public:
      using type = iterator;
    };

    template <>
    class out_edge_iterator< tag::vertex_list_graph >
    {
      using vertex_descriptor = vertex_descriptor_t<tag::vertex_list_graph>;
      using vertex_observer_collection = vertex_observer_collection_t<tag::vertex_list_graph>;
      using iterator = ::graph_model::edge::out_edge_iterator<vertex_descriptor,vertex_observer_collection>;
    public:
      using type = iterator;
    };

    template <>
    class out_edge_iterator< tag::vertex_and_edge_list_graph >
    {
      using vertex_descriptor = vertex_descriptor_t<tag::vertex_and_edge_list_graph>;
      using vertex_observer_collection = vertex_observer_collection_t<tag::vertex_and_edge_list_graph>;
      using iterator = ::graph_model::edge::out_edge_iterator<vertex_descriptor,vertex_observer_collection>;
    public:
      using type = iterator;
    };

    template < typename GraphTag >
    using out_edge_iterator_t = typename out_edge_iterator<GraphTag>::type;

  } // namespace traits

} // namespace graph_model

#endif
