/*
 * Copyright (c) 2017 Brook Milligan.
 *
 * This file is part of the GraphModel Library.
 *
 *     The GraphModel Library is free software: you can redistribute
 *     it and/or modify it under the terms of the GNU Affero General
 *     Public License as published by the Free Software Foundation,
 *     either version 3 of the License, or (at your option) any later
 *     version.
 *
 *     The GraphModel Library is distributed in the hope that it will
 *     be useful, but WITHOUT ANY WARRANTY; without even the implied
 *     warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
 *     PURPOSE.  See the GNU Affero General Public License for more
 *     details.
 *
 *     You should have received a copy of the GNU Affero General
 *     Public License along with the GraphModel Library.  If not, see
 *     <http://www.gnu.org/licenses/>.
 */

/// \file

#ifndef GRAPH_MODEL_TYPE_TRAITS_HAS_EQUAL_TO_HPP
#define GRAPH_MODEL_TYPE_TRAITS_HAS_EQUAL_TO_HPP 1

#include <graph_model/type_traits/void_t.hpp>

namespace graph_model
{

  namespace type_traits
  {

    template < typename, typename = void_t<> >
    struct has_equal_to
      : std::false_type
    {};

    /// \brief Does \c T provide equality comparison?
    ///
    template < typename T >
    struct has_equal_to < T
			  , void_t<decltype(std::declval<T>() == std::declval<T>())>
			  >
      : std::true_type
    {};

    /// \brief Does \c T provide equality comparison?
    ///
    template < typename T >
    using has_equal_to_t = typename has_equal_to<T>::type;

  } // namespace type_traits

} // namespace graph_model

#endif
