/*
 * Copyright (c) 2016,2017 Brook Milligan.
 *
 * This file is part of the GraphModel Library.
 *
 *     The GraphModel Library is free software: you can redistribute
 *     it and/or modify it under the terms of the GNU Affero General
 *     Public License as published by the Free Software Foundation,
 *     either version 3 of the License, or (at your option) any later
 *     version.
 *
 *     The GraphModel Library is distributed in the hope that it will
 *     be useful, but WITHOUT ANY WARRANTY; without even the implied
 *     warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
 *     PURPOSE.  See the GNU Affero General Public License for more
 *     details.
 *
 *     You should have received a copy of the GNU Affero General
 *     Public License along with the GraphModel Library.  If not, see
 *     <http://www.gnu.org/licenses/>.
 */

/// \file

#ifndef SAMPLE_SPACE_IS_POSSIBLE_HPP
#define SAMPLE_SPACE_IS_POSSIBLE_HPP 1

#include <limits>
#include <vector>

namespace sample_space
{

  namespace detail
  {

    struct is_possible
    {
      static constexpr bool apply (bool) { return true; }

      template < typename T >
      static bool apply (T t)
      {
	typedef std::numeric_limits<T> numeric_limits;
	// std::cout << "XXX - sample_space::is_possible(" << t << "): [" << numeric_limits::lowest() << "," << numeric_limits::max() << "]" << std::endl;
	return numeric_limits::lowest() <= t && t <= numeric_limits::max();
      }
    };

  } // namespace detail
  
#define DEFINE_IS_POSSIBLE(T)						\
  bool is_possible (T t)						\
  { return detail::is_possible::apply(t); }				\
  /**/

  DEFINE_IS_POSSIBLE(bool)
  DEFINE_IS_POSSIBLE(char)
  DEFINE_IS_POSSIBLE(signed char)
  DEFINE_IS_POSSIBLE(unsigned char)
  DEFINE_IS_POSSIBLE(wchar_t)
  DEFINE_IS_POSSIBLE(char16_t)
  DEFINE_IS_POSSIBLE(char32_t)
  DEFINE_IS_POSSIBLE(short int)
  DEFINE_IS_POSSIBLE(unsigned short int)
  DEFINE_IS_POSSIBLE(int)
  DEFINE_IS_POSSIBLE(unsigned int)
  DEFINE_IS_POSSIBLE(long int)
  DEFINE_IS_POSSIBLE(unsigned long int)
  DEFINE_IS_POSSIBLE(long long int)
  DEFINE_IS_POSSIBLE(unsigned long long int)
  DEFINE_IS_POSSIBLE(float)
  DEFINE_IS_POSSIBLE(double)
  DEFINE_IS_POSSIBLE(long double)

#undef DEFINE_IS_POSSIBLE

  template < typename T, typename Alloc >
  bool is_possible (std::vector<T,Alloc>) { return true; }
  
} // namespace sample_space

#endif
