/*
 * Copyright (c) 2017 Gregory Penn.
 *
 * This file is part of the GraphModel Library.
 *
 *     The GraphModel Library is free software: you can redistribute
 *     it and/or modify it under the terms of the GNU Affero General
 *     Public License as published by the Free Software Foundation,
 *     either version 3 of the License, or (at your option) any later
 *     version.
 *
 *     The GraphModel Library is distributed in the hope that it will
 *     be useful, but WITHOUT ANY WARRANTY; without even the implied
 *     warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
 *     PURPOSE.  See the GNU Affero General Public License for more
 *     details.
 *
 *     You should have received a copy of the GNU Affero General
 *     Public License along with the GraphModel Library.  If not, see
 *     <http://www.gnu.org/licenses/>.
 */

/// \file

#define BOOST_TEST_MODULE gdal

#ifdef INCLUDE_UNIT_TEST_HEADERS
#include <boost/test/included/unit_test.hpp>
#else
#include <boost/test/unit_test.hpp>
#endif

template < typename ... T > class XXX;

#include <iostream>
#include <memory>
#include <gdal/smartptr.hpp>
#include <gdal/rasterdataset.hpp>

using namespace gdal;

#define rasterfile "../data/test_raster.tif"

struct obj
{
  obj () = default;
  obj (int refs)
    : count(refs)
    {}

  ~obj () { f() = 0; }

  void Reference () { count = count + 1; }
  void Dereference () { count = count - 1; }

  int GetRefCount() { return count; }

  static int& f () { static int i(1); return i; }

  int count = 1;
};

BOOST_AUTO_TEST_CASE(default_constructor)
{
  auto p = smartptr<int>();
}

BOOST_AUTO_TEST_CASE(lambda_constructor)
{
  obj* p;
  p = new obj;

  smartptr<obj> sp(p
                 , [p] () { delete p; }
                 , [p] () { p->Reference(); }
                 , [p] () { p->Dereference(); }
                 , [p] () { return p->GetRefCount();});

  BOOST_CHECK_EQUAL(sp.use_count(), 2);
}

BOOST_AUTO_TEST_CASE(copy_constructor)
{
  obj* p;
  p = new obj;

  smartptr<obj> sp(p
                   , [p] () { delete p; }
                   , [p] () { p->Reference(); }
                   , [p] () { p->Dereference(); }
                   , [p] () { return p->GetRefCount();});

  std::cout << "sp.use_count(): " << sp.use_count() << std::endl;
  auto sp_copy = sp;
  std::cout << "sp.use_count(): " << sp.use_count() << std::endl;
  std::cout << "sp_copy.use_count(): " << sp_copy.use_count() << std::endl;

//  BOOST_CHECK_EQUAL(sp.use_count(), 2);

}
