/*
 * Copyright (c) 2016,2017 Brook Milligan.
 *
 * This file is part of the GraphModel Library.
 *
 *     The GraphModel Library is free software: you can redistribute
 *     it and/or modify it under the terms of the GNU Affero General
 *     Public License as published by the Free Software Foundation,
 *     either version 3 of the License, or (at your option) any later
 *     version.
 *
 *     The GraphModel Library is distributed in the hope that it will
 *     be useful, but WITHOUT ANY WARRANTY; without even the implied
 *     warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
 *     PURPOSE.  See the GNU Affero General Public License for more
 *     details.
 *
 *     You should have received a copy of the GNU Affero General
 *     Public License along with the GraphModel Library.  If not, see
 *     <http://www.gnu.org/licenses/>.
 */

#define BOOST_TEST_MODULE graph_model_distribution_normal_normal_strategy

#ifdef INCLUDE_UNIT_TEST_HEADERS
#include <boost/test/included/unit_test.hpp>
#else
#include <boost/test/unit_test.hpp>
#endif

template < typename ... T > class XXX;

#include <graph_model/distribution/normal/normal_strategy.hpp>

class parameter_pack {};

BOOST_AUTO_TEST_CASE(test_default_constructor)
{
  using tag = graph_model::vertex::tag::standard_deviation;
  using normal_strategy_type
    = graph_model::distribution::normal::normal_strategy<tag,parameter_pack>;
  normal_strategy_type strategy;
}

BOOST_AUTO_TEST_CASE(test_constructor_function)
{
  using tag = graph_model::distribution::parameter::tag::standard_deviation;
  using mean = graph_model::distribution::parameter::tag::mean;
  using dispersion = graph_model::distribution::parameter::tag::dispersion;
  using variate = graph_model::distribution::parameter::tag::variate;

  using boost::hana::make_map;
  using boost::hana::make_pair;
  using boost::hana::type_c;
  using graph_model::distribution::normal::make_normal_strategy;
  using graph_model::distribution::parameter::make_parameter;
  using graph_model::distribution::parameter::make_parameter_pack;
  using graph_model::vertex::make_expression_function;

  auto strategy = make_normal_strategy<tag>
    (make_parameter_pack
     (make_parameter<mean>().default_value(0),
      make_parameter<dispersion>().default_value(1),
      make_parameter<variate>().default_value(0)));
  strategy.sample();
  strategy.pdf();
  strategy.cdf();
}
